@@ -29,26 +29,37 @@
 
 #include <string.h>
 #include <map>
+#include <vector>
+#include<string>
 
-#include <plib/ul.h>
+#include <boost/foreach.hpp>
+#include "mat.hxx"
 
 #include <osg/CullFace>
 #include <osg/Material>
 #include <osg/ShadeModel>
+#include <osg/StateSet>
 #include <osg/TexEnv>
 #include <osg/Texture2D>
+#include <osgDB/ReaderWriter>
 #include <osgDB/ReadFile>
+#include <osgDB/Registry>
+#include <osgDB/FileUtils>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sgstream.hxx>
-
+#include <simgear/props/props_io.hxx>
 #include <simgear/scene/model/model.hxx>
+#include <simgear/scene/util/RenderConstants.hxx>
 #include <simgear/scene/util/StateAttributeFactory.hxx>
 
-#include "mat.hxx"
+#include "Effect.hxx"
+#include "Technique.hxx"
+#include "Pass.hxx"
 
 using std::map;
+using std::string;
 using namespace simgear;
 
 
@@ -56,76 +67,61 @@ using namespace simgear;
 // Constructors and destructor.
 ////////////////////////////////////////////////////////////////////////
 
-
-SGMaterial::SGMaterial( const string &fg_root, const SGPropertyNode *props, const char *season )
+SGMaterial::_internal_state::_internal_state(Effect *e, const string &t, bool l,
+                                             const osgDB::ReaderWriter::Options* o ) :
+  effect(e), texture_path(t), effect_realized(l), options(o)
 {
-    init();
-    read_properties( fg_root, props, season );
-    build_state( false );
 }
 
-SGMaterial::SGMaterial( const string &texpath )
+SGMaterial::SGMaterial( const osgDB::ReaderWriter::Options* options,
+                        const SGPropertyNode *props )
 {
     init();
-
-    _internal_state st( NULL, texpath, false );
-    _status.push_back( st );
-
-    build_state( true );
-}
-
-SGMaterial::SGMaterial( osg::StateSet *s )
-{
-    init();
-    set_state( s );
+    read_properties( options, props );
+    buildEffectProperties(options);
 }
 
 SGMaterial::~SGMaterial (void)
 {
 }
 
-
 
 ////////////////////////////////////////////////////////////////////////
 // Public methods.
 ////////////////////////////////////////////////////////////////////////
 
 void
-SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props, const char *season )
+SGMaterial::read_properties(const osgDB::ReaderWriter::Options* options,
+                            const SGPropertyNode *props)
 {
 				// Gather the path(s) to the texture(s)
   vector<SGPropertyNode_ptr> textures = props->getChildren("texture");
   for (unsigned int i = 0; i < textures.size(); i++)
   {
     string tname = textures[i]->getStringValue();
-    string otname = tname;
-
-    if (tname == "") {
+    if (tname.empty()) {
         tname = "unknown.rgb";
     }
-
-    SGPath tpath( fg_root );
-    tpath.append("Textures.high");
+    SGPath tpath("Textures.high");
     tpath.append(tname);
-    if ( !ulFileExists(tpath.c_str()) ) {
-      tpath = SGPath( fg_root );
-      tpath.append("Textures");
+    string fullTexPath = osgDB::findDataFile(tpath.str(), options);
+    if (fullTexPath.empty()) {
+      tpath = SGPath("Textures");
       tpath.append(tname);
+      fullTexPath = osgDB::findDataFile(tpath.str(), options);
     }
 
-    if ( ulFileExists(tpath.c_str()) ) {
-      _internal_state st( NULL, tpath.str(), false );
+    if (!fullTexPath.empty() ) {
+      _internal_state st( NULL, fullTexPath, false, options );
       _status.push_back( st );
     }
   }
 
   if (textures.size() == 0) {
-    string tname = "unknown.rgb";
-    SGPath tpath( fg_root );
-    tpath.append("Textures");
+    SGPath tpath("Textures");
     tpath.append("Terrain");
-    tpath.append(tname);
-    _internal_state st( NULL, tpath.str(), true );
+    tpath.append("unknown.rgb");
+    _internal_state st( NULL, tpath.str(), true, options );
     _status.push_back( st );
   }
 
@@ -135,15 +131,19 @@ SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props
   wrapv = props->getBoolValue("wrapv", true);
   mipmap = props->getBoolValue("mipmap", true);
   light_coverage = props->getDoubleValue("light-coverage", 0.0);
-  tree_coverage = props->getDoubleValue("tree-coverage", 0.0);
+  wood_coverage = props->getDoubleValue("wood-coverage", 0.0);
+  wood_size = props->getDoubleValue("wood-size", 0.0);
+  tree_density = props->getDoubleValue("tree-density", 1.0);
   tree_height = props->getDoubleValue("tree-height-m", 0.0);
   tree_width = props->getDoubleValue("tree-width-m", 0.0);
   tree_range = props->getDoubleValue("tree-range-m", 0.0);
   tree_varieties = props->getIntValue("tree-varieties", 1);
 
-  SGPath tpath( fg_root );
-  tpath.append(props->getStringValue("tree-texture"));
-  tree_texture = tpath.str();
+  const SGPropertyNode* treeTexNode = props->getChild("tree-texture");
+  if (treeTexNode) {
+    string treeTexPath = props->getStringValue("tree-texture");
+    tree_texture = osgDB::findDataFile(treeTexPath, options);
+  }
 
   // surface values for use with ground reactions
   solid = props->getBoolValue("solid", true);
@@ -175,6 +175,9 @@ SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props
 
   shininess = props->getDoubleValue("shininess", 1.0);
 
+  if (props->hasChild("effect"))
+      effect = props->getStringValue("effect");
+  
   vector<SGPropertyNode_ptr> object_group_nodes =
     ((SGPropertyNode *)props)->getChildren("object-group");
   for (unsigned int i = 0; i < object_group_nodes.size(); i++)
@@ -221,92 +224,64 @@ SGMaterial::init ()
         diffuse[i]  = (i < 3) ? 0.8 : 1.0;
         emission[i] = (i < 3) ? 0.0 : 1.0;
     }
+    effect = "Effects/terrain-default";
 }
 
-osg::StateSet *
-SGMaterial::get_state (int n)
+Effect* SGMaterial::get_effect(int n)
 {
     if (_status.size() == 0) {
-        SG_LOG( SG_GENERAL, SG_WARN, "No state available.");
-        return NULL;
+        SG_LOG( SG_GENERAL, SG_WARN, "No effect available.");
+        return 0;
     }
-    
     int i = n >= 0 ? n : _current_ptr;
-
-    if(!_status[i].texture_loaded)
-    {
-        assignTexture(_status[i].state.get(), _status[i].texture_path,
-                      wrapu, wrapv, mipmap);
-        _status[i].texture_loaded = true;
+    if(!_status[i].effect_realized) {
+        _status[i].effect->realizeTechniques(_status[i].options.get());
+        _status[i].effect_realized = true;
     }
-    osg::StateSet *st = _status[i].state.get();
-
-    _current_ptr += 1;
-    if (_current_ptr >= _status.size())
-        _current_ptr = 0;
-
-    return st;
+    // XXX This business of returning a "random" alternate texture is
+    // really bogus. It means that the appearance of the terrain
+    // depends on the order in which it is paged in!
+    _current_ptr = (_current_ptr + 1) % _status.size();
+    return _status[i].effect.get();
 }
 
-
-void 
-SGMaterial::build_state( bool defer_tex_load )
+void SGMaterial::buildEffectProperties(const osgDB::ReaderWriter::Options*
+                                       options)
 {
-    StateAttributeFactory *attrFact = StateAttributeFactory::instance();
-    for (unsigned int i = 0; i < _status.size(); i++)
+    using namespace osg;
+    ref_ptr<SGMaterialUserData> user = new SGMaterialUserData(this);
+    SGPropertyNode_ptr propRoot = new SGPropertyNode();
+    makeChild(propRoot, "inherits-from")->setStringValue(effect);
+    SGPropertyNode* paramProp = makeChild(propRoot, "parameters");
+    SGPropertyNode* materialProp = makeChild(paramProp, "material");
+    makeChild(materialProp, "ambient")->setValue(SGVec4d(ambient));    
+    makeChild(materialProp, "diffuse")->setValue(SGVec4d(diffuse));
+    makeChild(materialProp, "specular")->setValue(SGVec4d(specular));
+    makeChild(materialProp, "emissive")->setValue(SGVec4d(emission));
+    makeChild(materialProp, "shininess")->setFloatValue(shininess);
+    if (ambient[3] < 1 || diffuse[3] < 1 ||
+        specular[3] < 1 || emission[3] < 1) {
+        makeChild(paramProp, "transparent")->setBoolValue(true);
+        SGPropertyNode* binProp = makeChild(paramProp, "render-bin");
+        makeChild(binProp, "bin-number")->setIntValue(TRANSPARENT_BIN);
+        makeChild(binProp, "bin-name")->setStringValue("DepthSortedBin");
+    }
+    BOOST_FOREACH(_internal_state& matState, _status)
     {
-        osg::StateSet *stateSet = new osg::StateSet;
-        stateSet->setUserData(new SGMaterialUserData(this));
-
-        // Set up the textured state
-        stateSet->setAttribute(attrFact->getSmoothShadeModel());
-        stateSet->setAttributeAndModes(attrFact->getCullFaceBack());
-
-        stateSet->setMode(GL_LIGHTING, osg::StateAttribute::ON);
-
-        _status[i].texture_loaded = false;
-
-        osg::Material* material = new osg::Material;
-        material->setColorMode(osg::Material::AMBIENT_AND_DIFFUSE);
-        material->setAmbient(osg::Material::FRONT_AND_BACK, ambient.osg());
-        material->setDiffuse(osg::Material::FRONT_AND_BACK, diffuse.osg());
-        material->setSpecular(osg::Material::FRONT_AND_BACK, specular.osg());
-        material->setEmission(osg::Material::FRONT_AND_BACK, emission.osg());
-        material->setShininess(osg::Material::FRONT_AND_BACK, shininess );
-        stateSet->setAttribute(material);
-
-        if (ambient[3] < 1 || diffuse[3] < 1 ||
-            specular[3] < 1 || emission[3] < 1) {
-          stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
-          stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);
-          stateSet->setMode(GL_ALPHA_TEST, osg::StateAttribute::ON);
-        } else {
-          stateSet->setRenderingHint(osg::StateSet::OPAQUE_BIN);
-          stateSet->setMode(GL_BLEND, osg::StateAttribute::OFF);
-          stateSet->setMode(GL_ALPHA_TEST, osg::StateAttribute::OFF);
-        }
-
-        _status[i].state = stateSet;
+        SGPropertyNode_ptr effectProp = new SGPropertyNode();
+        copyProperties(propRoot, effectProp);
+        SGPropertyNode* effectParamProp = effectProp->getChild("parameters", 0);
+        SGPropertyNode* texProp = makeChild(effectParamProp, "texture");
+        makeChild(texProp, "image")->setStringValue(matState.texture_path);
+        makeChild(texProp, "filter")
+            ->setStringValue(mipmap ? "linear-mipmap-linear" : "nearest");
+        makeChild(texProp, "wrap-s")
+            ->setStringValue(wrapu ? "repeat" : "clamp");
+        makeChild(texProp, "wrap-t")
+            ->setStringValue(wrapv ? "repeat" : "clamp");
+        matState.effect = makeEffect(effectProp, false, options);
+        matState.effect->setUserData(user.get());
     }
-}
-
-
-void SGMaterial::set_state( osg::StateSet *s )
-{
-    _status.push_back( _internal_state( s, "", true ) );
-}
-
-void SGMaterial::assignTexture( osg::StateSet *state, const std::string &fname,
-                 bool _wrapu, bool _wrapv, bool _mipmap )
-{
-   osg::Texture2D* texture = SGLoadTexture2D(fname, 0, _wrapu, _wrapv,
-                                             mipmap ? -1 : 0);
-   texture->setMaxAnisotropy( SGGetTextureFilter());
-   state->setTextureAttributeAndModes(0, texture);
-
-   osg::TexEnv* texEnv = new osg::TexEnv;
-   texEnv->setMode(osg::TexEnv::MODULATE);
-   state->setTextureAttributeAndModes(0, texEnv);
 }
 
 SGMaterialGlyph* SGMaterial::get_glyph (const string& name) const
