@@ -26,6 +26,9 @@
 
 #include "extensions.hxx"
 #include <simgear/debug/logstream.hxx>
+#if !defined(WIN32)
+#  include <dlfcn.h>
+#endif
 
 bool SGSearchExtensionsString(const char *extString, const char *extName) {
     // Returns GL_TRUE if the *extName string appears in the *extString string,
@@ -53,7 +56,7 @@ bool SGSearchExtensionsString(const char *extString, const char *extName) {
     return GL_FALSE;
 }
 
-bool SGIsOpenGLExtensionSupported(char *extName) {
+bool SGIsOpenGLExtensionSupported(const char *extName) {
    // Returns GL_TRUE if the OpenGL Extension whose name is *extName
    // is supported by the system, or GL_FALSE otherwise.
    //
@@ -100,6 +103,7 @@ void* macosxGetGLProcAddress(const char *func) {
 
 void *SGGetGLProcAddress(const char *func) {
     static void *libHandle = NULL;
+    static void *(*glXGetProcAddressPtr)(const GLubyte*) = 0;
     void *fptr = NULL;
 
     /*
@@ -113,10 +117,32 @@ void *SGGetGLProcAddress(const char *func) {
      * arise from linking with a different libGL at link time an than later
      * use the standard libGL at runtime ...
      */
-    if (libHandle == NULL)
+    if (libHandle == NULL) {
         libHandle = dlopen(NULL, RTLD_LAZY);
 
-    if (libHandle != NULL) {
+        if (!libHandle) {
+#if defined (__FreeBSD__)
+            const char *error = dlerror();
+#else
+            char *error = dlerror();
+#endif
+            if (error) {
+                SG_LOG(SG_GENERAL, SG_INFO, error);
+                return 0;
+            }
+        }
+
+        void* symbol = dlsym(libHandle, "glXGetProcAddress");
+        if (!symbol)
+            symbol = dlsym(libHandle, "glXGetProcAddressARB");
+        glXGetProcAddressPtr = (void *(*)(const GLubyte*)) symbol;
+    }
+
+    // First try the glx api function for that
+    if (glXGetProcAddressPtr) {
+        fptr = glXGetProcAddressPtr((const GLubyte*)func);
+
+    } else if (libHandle != NULL) {
         fptr = dlsym(libHandle, func);
 
 #if defined (__FreeBSD__)
