@@ -27,11 +27,11 @@
 #include <vector>
 
 #include <map>
-#include STL_STRING
+#include <string>
 
-SG_USING_STD(map);
-SG_USING_STD(vector);
-SG_USING_STD(string);
+using std::map;
+using std::vector;
+using std::string;
 
 class Shader {
 public:
